//============================================================
//============================================================
//==														==
//==					Computer Header						==
//==														==
//============================================================
//============================================================


//========================================================  Defines  

#define kGeorgeCrouchPercent	60
#define kMaraCrouchPercent		70
#define kOttoCrouchPercent		80
#define kClaireCrouchPercent	90
#define kEazeCrouchPercent		96
#define kTeakCrouchPercent		98


//========================================================  Prototypes  

short DiagonalToPerson (playerType *);
short DiagonalToBall (playerType *);
short OctalToPoint (playerType *, register short, register short);
void LookUpBoardForce (playerType *);
Boolean NearTheBall (playerType *);
Boolean NearTheEdge (playerType *);
Boolean NearTheEdgeSmarter (playerType *);
Boolean NearTheGoal (playerType *);
Boolean NearPerson (void);
Boolean DecentShot (playerType *);
Boolean ExcellentShot (playerType *);
Boolean ExcellentDirectionalShot (playerType *, short);
Boolean PointingToDisaster (playerType *);
short RunTheDiagonalNE (playerType *);
short RunTheDiagonalNW (playerType *);
short RunTheDiagonalSmarterNE (playerType *);
short RunTheDiagonalSmarterNW (playerType *);
short RunTheDiagonalSmartestNE (playerType *);
short RunTheDiagonalSmartestNW (playerType *);
short SeekPlayerGoal (playerType *);
short AnticipateBall (playerType *, short);
short SeekPersonsGoal (playerType *);
short AnticipatePersonsFlank (playerType *, short);
short AnticipatePersonsFace (playerType *, short);
short RunTheCircleRight (playerType *);
short RunTheCircleLeft (playerType *);
void BrakeIfLeaving (playerType *);
void BashUnlessLeaving (playerType *);
void GeorgeDecides (playerType *);
void MaraDecides (playerType *);
void OttoDecides (playerType *);
void ClaireDecides (playerType *);
void EazeDecides (playerType *);
void TeakDecides (playerType *);


//========================================================  External Variables 

extern playerType	thePlayer;
extern playerType	theOpponent;
extern ballType		theBall;
extern long			oldDistSquared;
extern short		whosGotBall, arenaSize, loopsBallHeld;
extern short		*littleForceTable;
extern short		vertTable[41][81];			// temp testing 
extern short		displayHCenter;				// temp testing 
extern short		spacialToDirection[9];
extern short		boardForceTable[9][2];
extern Boolean		gameIsOver;

